const path = require("path");
const webpack = require("webpack");
const merge = require("webpack-merge");
const vueConfig = require("./vue-loader.config");
const ExtractTextPlugin = require("extract-text-webpack-plugin");
const FriendlyErrorsPlugin = require("friendly-errors-webpack-plugin");

let config = require("../config").getConfig();

const isProd = process.env.NODE_ENV === "production";

module.exports = merge(
  {
    devtool: isProd ? false : "#cheap-module-source-map",
    output: {
      path: path.resolve(config.projectPath, "./dist"),
      publicPath: "/dist/",
      filename: "[name].[chunkhash].js"
    },
    resolve: {
      alias: {
        public: path.resolve(config.projectPath, "./public")
      }
    },
    module: {
      noParse: /es6-promise\.js$/, // avoid webpack shimming process
      rules: [
        {
          test: /\.vue$/,
          loader: "vue-loader",
          options: vueConfig
        },
        {
          test: /\.js$/,
          loader: "babel-loader",
          exclude: /node_modules/
        },
        {
          test: /\.(png|jpg|gif|svg)$/,
          loader: "url-loader",
          options: {
            limit: 10000,
            name: "[name].[ext]?[hash]"
          }
        },
        {
          test: /\.css$/,
          use: isProd
            ? ExtractTextPlugin.extract({
                use: "css-loader?minimize",
                fallback: "vue-style-loader"
              })
            : ["vue-style-loader", "css-loader"]
        }
      ]
    },
    performance: {
      maxEntrypointSize: 300000,
      hints: isProd ? "warning" : false
    },
    plugins: isProd
      ? [
          new webpack.optimize.UglifyJsPlugin({
            compress: { warnings: false }
          }),
          new webpack.optimize.ModuleConcatenationPlugin(),
          new ExtractTextPlugin({
            filename: "common.[chunkhash].css"
          })
        ]
      : [new FriendlyErrorsPlugin()]
  },
  config.webpackConfig.base
);
